/* allgemeine Zeichen-I/O-Verwaltung */

/* wichtig:
hier werden auch solche I/O-Strme verwaltet, die es auf dem
konkreten System gar nicht gibt. Die enden dann im konkreten
Fall einfach blind.

 */

#include "StdTypes.h"
#include "gio.h"
#include "serial.h"
#include "usb.h"

word  StdIO;          /* bei stdin+stdout wird dieses Handle genommen */
char* TxBufPtr;       /* Schreibzeiger fr toBuf */
int   TxBufRemaining; /* Restlnge fr toBuf */


/* Zeichen in ein Kommandofenster o.. ausgeben */
bool CharToCommandWindow (char c)
{ return false;
}

/* Zeichen in eine Textdatei o.. ausgeben */
bool CharToFileStream(char c, word wo)
{ return false;
}

/* Zeichen ausgeben, ggf. mit Warten */
char Char_Out (char c, word wo)
{ word w;
  w = wo;
  if (w==stdout) w = StdIO;
  switch (w)
  { case  toUART0: return 0;
    case  toUART1: V24Char_Out1(c); return c;
    case  toUART2: V24Char_Out2(c); return c;
    case  toUART3: V24Char_Out3(c); return c;
    case  toUART4: V24Char_Out4(c); return c;
    case  toUART5: V24Char_Out5(c); return c;
    case  toUART6: return 0;
    case  toUART7: return 0;
    case  toUSB:   UsbCharOut(c);   return c;
    case  toCmd:   if (CharToCommandWindow(c)) return c;
                   return 0;
    case  toBUF:
      if (TxBufRemaining < 1) return 0;
      *TxBufPtr++ = c;
      *TxBufPtr   = 0;
      --TxBufRemaining;
      return c;

    default:
    if (CharToFileStream(c, wo)) return c;
  }
 return 0;
}


/* String  ausgeben  */
void String_Out (char* P, word wo)
{ while (*P) Char_Out(*P++, wo); }


/* CR und LF auf USART0 ausgeben  */
void CRLF_Out (word wo)
{ Char_Out(13, wo);
  Char_Out(10, wo);
}


/* Anzahl freier Pltze im Stream ermitteln  */
int numTxFree (word wo)
{ word w;
  w = wo;
  if (w==stdout) w = StdIO;
  switch (w)
  { case  toUART0: return 0;
    case  toUART1: return V24numTxFree1();
    case  toUART2: return V24numTxFree2();
    case  toUART3: return V24numTxFree3();
    case  toUART4: return V24numTxFree4();
    case  toUART5: return V24numTxFree5();
    case  toUART6: return 0;
    case  toUART7: return 0;
    case  toUSB:   return UsbTxFree();
    case  toCmd:   return 1;
    case  toBUF:   return TxBufRemaining;
    default:  return -1;
  }
}


/* Test, ob Zeichen vom Stream gelesen werden knnen */
bool RxAvail (word wo)
{ word w;
  w = wo;
  if (w==stdout) w = StdIO;
  switch (w)
  { case  toUART0: return false;
    case  toUART1: return V24RxAvail1();
    case  toUART2: return V24RxAvail2();
    case  toUART3: return V24RxAvail3();
    case  toUART4: return V24RxAvail4();
    case  toUART5: return V24RxAvail5();
    case  toUART6: return false;
    case  toUART7: return false;
    case  toUSB:   return UsbRxAvail();
    case  toCmd:   return false;
    case  toBUF:   return false;
    default:  return false;
  }
}

/* Zeichen vom Stream empfangen */
char GetChar (word wo)
{ word w;
  w = wo;
  if (w==stdout) w = StdIO;
  switch (w)
  { case  toUART0: return 0;
    case  toUART1: return V24GetChar1();
    case  toUART2: return V24GetChar2();
    case  toUART3: return V24GetChar3();
    case  toUART4: return V24GetChar4();
    case  toUART5: return V24GetChar5();
    case  toUART6: return 0;
    case  toUART7: return 0;
    case  toUSB:   return UsbGetChar();
    case  toCmd:   return 0;
    case  toBUF:   return 0;
    default:  return 0;
  }
}

/* Warten, bis alle Zeichen gesendet sind
   ======================================
 */
void TxDone (word wo)
{ word w;
  w = wo;
  if (w==stdout) w = StdIO;
  switch (w)
  { case  toUART0: return;
    case  toUART1: V24TxDone1(); return;
    case  toUART2: V24TxDone2(); return;
    case  toUART3: V24TxDone3(); return;
    case  toUART4: V24TxDone4(); return;
    case  toUART5: V24TxDone5(); return;
    case  toUART6: return;
    case  toUART7: return;
    case  toUSB:   return;
    case  toCmd:   return;
    case  toBUF:   return;
    default:  return;
  }
}

 /* ende */
